﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Smile;

namespace SmileForm
{
    public partial class SmileForm : Form
    {
        public SmileForm()
        {
            InitializeComponent();
        }

        private void SmileForm_Load(object sender, System.EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "XDSL Files|*.xdsl";
            if (ofd.ShowDialog(this) == DialogResult.OK)
            {
                FillTreeView(ofd.FileName);
                RefreshBeliefs();
            }
            else
            {
                Close();
            }
        }

        private void RefreshBeliefs()
        {
            net.UpdateBeliefs();
            TreeNode root = treeView.Nodes[0];
            foreach (String id in net.GetAllNodeIds())
            {
                TreeNode treeNode = root.Nodes[id];
                double[] beliefs = null;
                String evidence = null;
                if (net.IsEvidence(id)) 
                {
                    evidence = net.GetEvidenceId(id);
                }
                else if (net.IsValueValid(id))
                {
                    beliefs = net.GetNodeValue(id);
                }

                int idx = 0;
                foreach (String state in net.GetOutcomeIds(id))
                {
                    String caption = state;
                    if (beliefs != null)
                    {
                        caption += String.Format(" = {0:F4}", beliefs[idx++]);
                    }
                    else
                    {
                        if (evidence != null && state.Equals(evidence))
                        {
                            caption += " [EVIDENCE]";
                        }
                    }
                    treeNode.Nodes[state].Text = caption;
                }
            }
        }

        private void FillTreeView(String filename)
        {
            net.ReadFile(filename);

            TreeNode root = treeView.Nodes.Add(filename);
            foreach (String id in net.GetAllNodeIds())
            {
                TreeNode treeNode = root.Nodes.Add(id, net.GetNodeName(id));
                if (net.IsTarget(id))
                {
                    treeNode.Text += " [TARGET]";
                }
                treeNode.Tag = id;
                foreach (String state in net.GetOutcomeIds(id))
                {
                    TreeNode stateNode = treeNode.Nodes.Add(state, state);
                    stateNode.Tag = state;
                }
            }

            root.ExpandAll();
            root.EnsureVisible();
        }


        private Network net = new Network();
        private TreeNode rightClickedNode;

        private void treeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                treeView.SelectedNode = e.Node;
                rightClickedNode = e.Node;
                Point p = new Point(e.X, e.Y);

                ContextMenuStrip ctx = null;
                switch (e.Node.Level)
                {
                    case 0:
                        ctx = rootMenu;
                        break;
                    case 1:
                        ctx = nodeMenu;
                        clearNodeEvidenceMenuItem.Enabled = net.IsEvidence(e.Node.Tag.ToString());
                        break;
                    case 2:
                        ctx = stateMenu;
                        String nodeId = e.Node.Parent.Tag.ToString();
                        String stateId = e.Node.Tag.ToString();
                        bool stateIsEvidence = net.IsEvidence(nodeId) && stateId.Equals(net.GetEvidenceId(nodeId));
                        clearStateEvidenceMenuItem.Enabled = stateIsEvidence;
                        setEvidenceMenuItem.Enabled = !stateIsEvidence;
                        break;
                }

                ctx.Show(this, p);
            }
        }

        private void clearEvidence_Click(object sender, EventArgs e)
        {
            Object nodeTag;
            if (rightClickedNode.Level == 1)
            {
                nodeTag = rightClickedNode.Tag;
            }
            else
            {
                nodeTag = rightClickedNode.Parent.Tag;
            }
            net.ClearEvidence(nodeTag.ToString());
            RefreshBeliefs();
        }

        private void setEvidence_Click(object sender, EventArgs e)
        {
            Object nodeTag = rightClickedNode.Parent.Tag;
            net.SetEvidence(nodeTag.ToString(), rightClickedNode.Index);
            RefreshBeliefs();
        }

        private void clearAllEvidence_Click(object sender, EventArgs e)
        {
            net.ClearAllEvidence();
            RefreshBeliefs();
        }
    }
}
